from .subcorpora import SubcorpusCollection


class CorpusCollection:
    def __init__(self, client):
        self.client = client

    def get(self, name):
        return Corpus(self.client, name)

    def list(self):
        return [Corpus(self.client, corpus) for corpus
                in self.client.corpus_list_coprora()]


class Corpus:
    def __init__(self, client, name):
        self.client = client
        self.name = name
        self.__load()

    def __load(self):
        self.size = self.client.cl_attribute_size('{}.word'.format(self.name))
        # self.info = client.corpus_info(self.name)
        self.charset = self.client.corpus_charset(self.name)
        # self.full_name = self.client.corpus_full_name(self.name)
        self.properties = self.client.corpus_properties(self.name)
        self.alignment_attributes = \
            self.client.corpus_alignment_attributes(self.name)
        self.structural_attributes = \
            self.client.corpus_structural_attributes(self.name)
        self.positional_attributes = \
            self.client.corpus_positional_attributes(self.name)
        self.subcorpora = SubcorpusCollection(self.client, self)

    def alg2cpos(self, attribute, alg):
        __attribute = '{}.{}'.format(self.name, attribute)
        return self.client.cl_alg2cpos(__attribute, alg)

    def cpos2alg(self, attribute, cpos_list):
        __attribute = '{}.{}'.format(self.name, attribute)
        return self.client.cl_cpos2alg(__attribute, cpos_list)

    def cpos2id(self, attribute, cpos_list):
        __attribute = '{}.{}'.format(self.name, attribute)
        return self.client.cl_cpos2id(__attribute, cpos_list)

    def cpos2lbound(self, attribute, cpos_list):
        __attribute = '{}.{}'.format(self.name, attribute)
        return self.client.cl_cpos2lbound(__attribute, cpos_list)

    def cpos2rbound(self, attribute, cpos_list):
        __attribute = '{}.{}'.format(self.name, attribute)
        return self.client.cl_cpos2rbound(__attribute, cpos_list)

    def cpos2str(self, attribute, cpos_list):
        __attribute = '{}.{}'.format(self.name, attribute)
        return self.client.cl_cpos2str(__attribute, cpos_list)

    def cpos2struc(self, attribute, cpos_list):
        __attribute = '{}.{}'.format(self.name, attribute)
        return self.client.cl_cpos2struc(__attribute, cpos_list)

    def id2cpos(self, attribute, id):
        __attribute = '{}.{}'.format(self.name, attribute)
        return self.client.cl_id2cpos(__attribute, id)

    def idlist2cpos(self, attribute, ids):
        __attribute = '{}.{}'.format(self.name, attribute)
        return self.client.cl_idlist2cpos(__attribute, ids)

    def id2freq(self, attribute, ids):
        __attribute = '{}.{}'.format(self.name, attribute)
        return self.client.cl_id2freq(__attribute, ids)

    def id2str(self, attribute, ids):
        __attribute = '{}.{}'.format(self.name, attribute)
        return self.client.cl_id2str(__attribute, ids)

    def query(self, query, subcorpus_name='Results'):
        self.client.cqp_query(self.name, subcorpus_name, query)
        return self.subcorpora.get('Results')

    def regex2id(self, attribute, regex):
        __attribute = '{}.{}'.format(self.name, attribute)
        return self.client.cl_regex2id(__attribute, regex)

    def structural_attribute_has_values(self, attribute):
        __attribute = '{}.{}'.format(self.name, attribute)
        return self.client.corpus_structural_attribute_has_values(__attribute)

    def str2id(self, attribute, strings):
        __attribute = '{}.{}'.format(self.name, attribute)
        return self.client.cl_str2id(__attribute, strings)

    def struc2cpos(self, attribute, struc):
        __attribute = '{}.{}'.format(self.name, attribute)
        return self.client.cl_struc2cpos(__attribute, struc)

    def struc2str(self, attribute, strucs):
        __attribute = '{}.{}'.format(self.name, attribute)
        return self.client.cl_struc2str(__attribute, strucs)
