from ..constants import (CONST_FIELD_KEYWORD, CONST_FIELD_MATCH,
                         CONST_FIELD_MATCHEND, CONST_FIELD_TARGET)


class SubcorpusCollection:
    def __init__(self, client, parent_corpus):
        self.client = client
        self.parent_corpus = parent_corpus

    def get(self, name):
        return Subcorpus(self.client, self.parent_corpus, name)

    def list(self):
        return [Subcorpus(self.client, self.parent_corpus, subcorpus)
                for subcorpus
                in self.client.cqp_list_subcorpora(self.parent_corpus.name)]


class Subcorpus:
    def __init__(self, client, parent_corpus, name):
        self.client = client
        self.parent_corpus = parent_corpus
        self.name = name
        self.__name = '{}:{}'.format(parent_corpus.name, name)
        self.__load()

    def __load(self):
        self.fields = {}
        if self.client.cqp_subcorpus_has_field(self.__name, CONST_FIELD_MATCH):
            self.fields['match'] = CONST_FIELD_MATCH
        if self.client.cqp_subcorpus_has_field(self.__name,
                                               CONST_FIELD_MATCHEND):
            self.fields['matchend'] = CONST_FIELD_MATCHEND
        if self.client.cqp_subcorpus_has_field(self.__name,
                                               CONST_FIELD_TARGET):
            self.fields['target'] = CONST_FIELD_TARGET
        if self.client.cqp_subcorpus_has_field(self.__name,
                                               CONST_FIELD_KEYWORD):
            self.fields['keyword'] = CONST_FIELD_KEYWORD
        self.size = self.client.cqp_subcorpus_size(self.__name)

    def drop(self):
        return self.client.cqp_drop_subcorpus(self.__name)

    def dump(self, field, first, last):
        return self.client.cqp_dump_subcorpus(self.__name, field, first, last)

    def dump_values(self, context=25, first_result=0,
                    num_results=float('inf')):
        if self.size == 0:
            return {"matches": []}
        first_result = max(0, first_result)
        last_result = min((first_result + num_results - 1), (self.size - 1))
        matches = []
        match_boundaries = zip(self.dump(self.fields['match'], first_result,
                                         last_result),
                               self.dump(self.fields['matchend'], first_result,
                                         last_result))
        for match_start, match_end in match_boundaries:
            left_start = max(0, match_start - context)
            right_end = min(self.parent_corpus.size, (match_end + 1 + context))
            matches.append({'lc': list(range(left_start, match_start)),
                            'hit': list(range(match_start, match_end + 1)),
                            'rc': list(range(match_end + 1, right_end))})
        cpos_list = []
        for match in matches:
            cpos_list += match['lc'] + match['hit'] + match['rc']
        cpos_list = list(set(cpos_list))
        lookups = {}
        if len(cpos_list) > 0:
            lookups['cpos_lookup'] = {}
        for cpos in cpos_list:
            lookups['cpos_lookup'][cpos] = {}
        for attr in self.parent_corpus.positional_attributes:
            cpos_attr_values = self.parent_corpus.cpos2str(attr, cpos_list)
            for i, cpos in enumerate(cpos_list):
                lookups['cpos_lookup'][cpos][attr] = cpos_attr_values[i]
        for attr in self.parent_corpus.structural_attributes:
            if self.parent_corpus.structural_attribute_has_values(attr):
                continue
            cpos_attr_ids = self.parent_corpus.cpos2struc(attr, cpos_list)
            for i, cpos in enumerate(cpos_list):
                if cpos_attr_ids[i] != -1:
                    lookups['cpos_lookup'][cpos][attr] = cpos_attr_ids[i]
            occured_attr_ids = list(set(cpos_attr_ids))
            occured_attr_ids = list(filter(lambda x: x != -1,
                                           occured_attr_ids))
            if len(occured_attr_ids) == 0:
                continue
            attr_subattrs = \
                list(filter(lambda x: x.startswith(attr + '_'),
                            self.parent_corpus.structural_attributes))
            attr_subattrs = list(map(lambda x: x.split('_', 1)[1],
                                     attr_subattrs))
            if len(attr_subattrs) == 0:
                continue
            lookups[attr + '_lookup'] = {}
            for attr_id in occured_attr_ids:
                lookups[attr + '_lookup'][attr_id] = {}
            for subattr in attr_subattrs:
                __subattr = attr + '_' + subattr
                subattr_values = \
                    self.parent_corpus.struc2str(__subattr, occured_attr_ids)
                for i, value in enumerate(subattr_values):
                    lookups[attr + '_lookup'][occured_attr_ids[i]][subattr] = \
                        value
        return {'matches': matches, **lookups}

    def fdist_1(self, cutoff, field, attribute):
        return self.client.cqp_fdist_1(self.__name, cutoff, field, attribute)

    def fdist_2(self, cutoff, field_1, attribute_1, field_2, attribute_2):
        return self.client.cqp_fdist_2(self.__name, cutoff, field_1,
                                       attribute_1, field_2, attribute_2)
